package fastly

import (
	"log"

	"github.com/spf13/cobra"
	"gitlab.alpinelinux.org/alpine/go/pkg/releases"
	"gitlab.alpinelinux.org/alpine/go/pkg/repository"
	"gitlab.alpinelinux.org/alpine/infra/repo-tools/internal/utils"
)

type PkgArgs struct {
	origin   bool
	filename bool

	releaseName    string
	repositoryName string
	arch           string
}

var (
	pkgArgs PkgArgs
)

func init() {
	purgePkgCmd := &cobra.Command{
		Use:   "pkg [--origin] <pkg..>",
		Short: "Purge specific packages",
		Args:  cobra.MinimumNArgs(1),
		Run:   fastlyPurgePkgCmd,
	}

	purgePkgCmd.Flags().BoolVar(&pkgArgs.origin, "origin", false, "Purge packages by origin")
	purgePkgCmd.Flags().BoolVar(&pkgArgs.filename, "filename", false, "Purge packages by package filename")

	purgePkgCmd.Flags().StringVar(&pkgArgs.releaseName, "release", "edge", "What release to purge the package for")
	purgePkgCmd.Flags().StringVar(&pkgArgs.repositoryName, "repo", "main", "What repository the package is in")
	purgePkgCmd.Flags().StringVar(&pkgArgs.arch, "arch", "", "Limit purging to a single arch only")

	purgeCmd.AddCommand(purgePkgCmd)
}

func fastlyPurgePkgCmd(cmd *cobra.Command, args []string) {
	alpineReleases, err := releases.Fetch()

	if err != nil {
		utils.Fatalf("Error fetching releases: %s\n", err)
	}

	release := alpineReleases.GetRelBranch(pkgArgs.releaseName)
	if release == nil {
		utils.Fatalf("Error: unknown release %s\n", pkgArgs.releaseName)
	}

	repositories := []*repository.RepositoryWithIndex{}

	var arches []string
	if pkgArgs.arch != "" {
		if !utils.ListContains(release.Arches, pkgArgs.arch) {
			utils.Fatalf("Unknown arch %s\n", pkgArgs.arch)
		}
		arches = []string{pkgArgs.arch}
	} else {
		arches = release.Arches
	}

	for _, arch := range arches {
		repo := repository.NewRepositoryFromComponents(mirror, pkgArgs.releaseName, pkgArgs.repositoryName, arch)
		indexFile, err := utils.FetchFile(repo.IndexUri())

		if err != nil {
			log.Printf("ERROR: fetching index for %s: %s", arch, err)
			continue
		}

		index, err := repository.IndexFromArchive(indexFile.Stream)

		if err != nil {
			log.Printf("ERROR: parsing index for %s: %s", arch, err)
			continue
		}

		repositories = append(repositories, repo.WithIndex(index))
	}

	packages := []*repository.RepositoryPackage{}
	selectedPackages := map[string]interface{}{}
	for _, pkg := range args {
		selectedPackages[pkg] = true
	}

	switch {
	case pkgArgs.origin:
		for _, repository := range repositories {
			for _, pkg := range repository.Packages() {
				if _, selected := selectedPackages[pkg.Origin]; selected {
					packages = append(packages, pkg)
				}
			}
		}
	case pkgArgs.filename:
		for _, repository := range repositories {
			for _, pkg := range repository.Packages() {
				if _, selected := selectedPackages[pkg.Filename()]; selected {
					packages = append(packages, pkg)
				}
			}
		}
	default:
		utils.Fatalf("Nothing to do\n")
	}

	Purge(packages, 2)
}
